(defsystem "lisp-sentry"
  :version "0.2.0"
  :author "Andrew Kravchuk"
  :mailto "awkravchuk@gmail.com"
  :license "MIT"
  :homepage "https://gitlab.com/lockie/lisp-sentry"
  :depends-on (#:alexandria
               #:asdf
               #:bordeaux-threads
               #:cffi
               #:cl-ppcre
               #:closer-mop
               #:drakma
               (:feature :lisp-sentry-gpu #:framebuffers)
               #:frugal-uuid
               #:local-time
               #:machine-state
               (:feature :lisp-sentry-gpu #:machine-state/opengl)
               #:salza2
               #:trivial-features
               #:trivial-mimes
               #:uiop)
  :components
  ((:module "src"
    :components
    ((:file "package")
     (:file "app"          :pathname "contexts/app"           :depends-on ("contexts" "helpers"))
     (:file "defaults"     :pathname "contexts/defaults"      :depends-on ("app" "device" "feature-flag" "gpu" "helpers" "os" "runtime"))
     (:file "device"       :pathname "contexts/device"        :depends-on ("event" "contexts" "helpers"))
     (:file "feature-flag" :pathname "contexts/feature-flag"  :depends-on ("contexts" "helpers"))
     (:file "gpu"          :pathname "contexts/gpu"           :depends-on ("contexts" "helpers"))
     (:file "os"           :pathname "contexts/os"            :depends-on ("contexts" "helpers"))
     (:file "runtime"      :pathname "contexts/runtime"       :depends-on ("contexts" "helpers"))
     (:file "sbcl"         :pathname "impl/sbcl"              :depends-on ("pretty-print" "source" "stack-trace") :if-feature :sbcl)
     (:file "breadcrumbs"  :pathname "interfaces/breadcrumbs" :depends-on ("event" "helpers"))
     (:file "contexts"     :pathname "interfaces/contexts"    :depends-on ("event" "helpers"))
     (:file "debug-meta"   :pathname "interfaces/debug-meta"  :depends-on ("event" "helpers"))
     (:file "exception"    :pathname "interfaces/exception"   :depends-on ("event" "helpers" "stack-trace"))
     (:file "message"      :pathname "interfaces/message"     :depends-on ("event" "helpers"))
     (:file "request"      :pathname "interfaces/request"     :depends-on ("event" "helpers"))
     (:file "sdk"          :pathname "interfaces/sdk"         :depends-on ("event" "helpers"))
     (:file "stack-trace"  :pathname "interfaces/stack-trace" :depends-on ("helpers"))
     (:file "threads"      :pathname "interfaces/threads"     :depends-on ("event" "helpers" "stack-trace"))
     (:file "user"         :pathname "interfaces/user"        :depends-on ("event" "helpers"))
     (:file "attachment"   :pathname "items/attachment"       :depends-on ("envelope" "helpers"))
     (:file "event"        :pathname "items/event"            :depends-on ("envelope" "helpers"))
     (:file "user-report"  :pathname "items/user-report"      :depends-on ("envelope" "helpers"))
     (:file "envelope"                                       :depends-on ("helpers" "payload"))
     (:file "helpers")
     (:file "lisp-sentry"                                    :depends-on ("app" "breadcrumbs" "contexts" "debug-meta" "defaults" "envelope" "event" "exception" "message" "sdk" "source" "stack-trace" "threads"))
     (:file "payload")
     (:file "pretty-print")
     (:file "source"))))
  :description "A full-featured Common Lisp client library for Sentry application monitoring software."
  :in-order-to ((test-op (test-op "lisp-sentry/tests"))))

(defsystem "lisp-sentry/tests"
  :author "Andrew Kravchuk"
  :license "MIT"
  :depends-on (#:lisp-sentry
               #:parachute
               (:feature :sbcl #:sb-cover))
  :serial t
  :components ((:module "tests"
                :components
                ((:file "package")
                 (:file "main"))))
  :description "Test system for lisp-sentry"
  :perform (test-op (op c) (uiop:symbol-call :lisp-sentry/tests '#:run)))
