(in-package #:lisp-sentry)


(declaim (type (or string null)
               *build-type* *app-name* *app-version* *app-build*))
(defvar *build-type* nil
  "Default string identifying the kind of build. For example, `testflight`.")
(defvar *app-name* nil
  "Default human readable application name, as it appears on the platform.")
(defvar *app-version* nil
  "Default human readable application version, as it appears on the platform.")
(defvar *app-build* nil
  "Default internal build identifier, as it appears on the platform.")

(defvar *startup-time* nil
  "Timestamp when the user started the application.")

(define-json-object (app-context)
  "[App context](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts/#app-context)
describes the application. As opposed to the runtime, this is the actual
application that was running and carries metadata about the current session.

See `MAKE-APP-CONTEXT`"
  (app-start-time
   (when *startup-time*
     (local-time:to-rfc3339-timestring *startup-time*))
   :type string :optional t
   :documentation
   "timestamp when the user started the application. See `*STARTUP-TIME*`.")
  (build-type *build-type* :type string :optional t
                           :documentation "string identifying the kind of build.
See `*BUILD-TYPE*`.")
  (app-identifier (pathname-name (or (uiop:argv0) ""))
                  :type string :optional t
                  :documentation "version-independent application identifier.
Calculated automatically using `UIOP:ARGV0`")
  (app-name *app-name* :type string :optional t
                       :documentation "human readable appication name, as it
appears on the platform. See `*APP-NAME*`.")
  (app-version *app-version* :type string :optional t
                             :documentation "human readable application version,
as it appears on the platform. See `*APP-VERSION*`.")
  (app-build *app-build* :type string :optional t
                         :documentation "internal build identifier, as it
appears on the platform. See `*APP-BUILD*`.")
  (view-names nil :type list :optional t :format "~{~s~^,~}"
                  :documentation "list of visible UI screens at the current
point in time.")
  (app-memory (machine-state:process-room)
              :type array-length
              :documentation "optional amount of memory used by the application
in bytes. Calculated automatically using `ROOM`."))

(defmethod dump-context ((context app-context))
  `("app"
    ,(dump-app-context context)))
