(in-package #:lisp-sentry)


(define-json-object (device-context)
  "[Device context](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts/#device-context)
describes the device that caused the event.

See `MAKE-DEVICE-CONTEXT`"
  (name *server-name* :type string :optional t
        :documentation "name of the device. This is typically a hostname.
See `*SERVER-NAME*`.")
  (family nil :type string :optional t
              :documentation "family of the device. This is usually the common
part of model names across generations. For instance, `iPhone` would be a
reasonable family, so would be `Samsung Galaxy`.")
  (arch nil :type string-designator :optional t :format "\"~(~a~)\""
            :documentation "CPU architecture.")
  (battery-level nil :type double-float :optional t :format "~$"
                 :documentation "floating point value defining the battery level
(in the range 0-100), if the device has a battery.")
  (manufacturer nil :type string :optional t
                    :documentation "manufacturer of the device.")
  (brand nil :type string :optional t :documentation "brand of the device.")
  (screen-width-pixels nil :type array-length :optional t
                           :documentation "width of the screen.")
  (screen-height-pixels nil :type array-length :optional t
                        :documentation "height of the screen.")
  (screen-resolution nil :type string :optional t
                         :documentation
                         "screen resolution (e.g.: 800x600, 3040x1444).")
  (charging nil :type boolean :format "~:[false~;true~]"
                :documentation "boolean flag indicating whether the device was
charging or not.")
  (memory-size 0 :type non-negative-fixnum
                 :documentation "total system memory available in bytes.")
  (free-memory 0 :type non-negative-fixnum
                 :documentation "free system memory in bytes.")
  (storage-size 0 :type non-negative-fixnum :optional t
                  :documentation "total device storage in bytes.")
  (free-storage 0 :type non-negative-fixnum :optional t
                  :documentation "free device storage in bytes.")
  (boot-time
   (princ-to-string (local-time:universal-to-timestamp
                     (- (get-universal-time) (machine-state:machine-uptime))))
   :type string :optional t
   :documentation "timestamp when the system was booted.")
  (processor-count
   (machine-state:machine-cores)
   :type array-length
   :documentation "number of \"logical processors\". For example, 8.")
  (cpu-description nil :type string :optional t
                       :documentation "CPU description. For example,
`Intel(R) Core(TM)2 Quad CPU Q6600 @ 2.40GHz`.")
  (battery-status nil :type string :optional t
                      :documentation "status of the device's battery.
For example, `Unknown`, `Charging`, `Discharging`, `NotCharging`, `Full`.")
  (device-type "Unknown"
               :type string
               :documentation "kind of device the application is running on.
For example, `Unknown`, `Handheld`, `Console`, `Desktop`."))

(defmethod dump-context ((context device-context))
  `("device"
    ,(dump-device-context context)))
