(in-package #:lisp-sentry)


(define-json-object (feature-flag-context)
  "The [feature flag context](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts/#feature-flag-context)
contains information about the flags evaluated prior to an error occurring. It
is populated with `*FEATURES*` by default.

See `MAKE-FEATURE-FLAG-CONTEXT`"
  (values *features* :type list
                     :format "[~{{\"flag\":\"~(~a~)\",\"result\":true}~^,~}]"
                     :processor (lambda (features)
                                  (mapcar
                                   (lambda (feature)
                                     (cond ((keywordp feature) feature)
                                           ((symbolp feature)
                                            (format nil "~a::~a"
                                                    (package-name
                                                     (symbol-package feature))
                                                    feature))
                                           (t feature)))
                                   features))
                     :documentation "the list of string designators
corresponding to enabled feature flags. Defaults to `*FEATURES*`."))

(defmethod dump-context ((context feature-flag-context))
  `("flags"
    ,(dump-feature-flag-context context)))
