(in-package #:lisp-sentry)


(define-json-object (gpu-context :ctor %make-gpu-context)
  "[GPU context](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts/#gpu-context)
describes the GPU of the device.

See `MAKE-GPU-CONTEXT`"
  (version nil :type string :optional t)
  (vendor-name nil :type string-designator :optional t :format "\"~:(~a~)\"")
  (max-texture-size nil :type fixnum :optional t)
  (memory-size nil :type non-negative-fixnum :optional t)
  (name "Unknown" :type string))

(defun make-gpu-context ()
  "Creates `GPU-CONTEXT`.

**NOTE**: this works in a meaningful way only when `:LISP-SENTRY-GPU` was present
in `*FEATURES*` when the system was compiled and there is active OpenGL
context in current thread."
  (apply #'%make-gpu-context
         #+lisp-sentry-gpu
         (multiple-value-bind (vendor model version)
             (ignore-errors* (machine-state:gpu-info))
           (multiple-value-bind (free total)
               (ignore-errors* (machine-state:gpu-room))
             (declare (ignore free))
             `(:name ,(or model "Unknown")
               :version ,version
               :vendor-name ,vendor
               :memory-size
               ,(when total
                  (truncate total (* 1024 1024)))
               :max-texture-size
               ,(ignore-errors
                 (gl:get-integer :max-texture-size)))))
         #-lisp-sentry-gpu
         nil))

(defmethod dump-context ((context gpu-context))
  `("gpu"
    ,(dump-gpu-context context)))
