(in-package #:lisp-sentry)


(define-json-object (os-context)
  "[OS context](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts/#os-context)
describes the operating system on which the crash happened/the event was
created.

See `MAKE-OS-CONTEXT`"
  (distribution-name
   nil
   :type string :optional t
   :documentation "stable name for each distribution. This maps to ID in
[`/etc/os-release`](https://www.freedesktop.org/software/systemd/man/latest/os-release.html#ID=)
(examples: `ubuntu`, `rhel`, `alpine`; a full list of tested identifiers is
available in the
[Native SDK repository](https://github.com/getsentry/sentry-native/blob/master/tests/fixtures/os_releases/distribution_names.txt)).")
  (distribution-version
   nil
   :type string :optional t
   :documentation "identifier of at least the major release version number.
This maps to `VERSION_ID` in
[`/etc/os-release`](https://www.freedesktop.org/software/systemd/man/latest/os-release.html#ID=).
Distributions with rolling releases only, will not provide a version.")
  (distribution-pretty-name
   nil
   :type string :optional t
   :documentation "full name, full version, and release alias. This maps to
`PRETTY_NAME` in
[`/etc/os-release`](https://www.freedesktop.org/software/systemd/man/latest/os-release.html#ID=)
(examples: `Ubuntu 22.04.4 LTS`, `Raspian GNU/Linux 10 (buster)`).")
  (name "" :type string-designator :format "\"~@(~a~)\""
           :documentation "the name of the operating system.")
  (version "" :type string
              :documentation "the version of the operating system."))

(defmethod dump-context ((context os-context))
  `("os"
    ,(dump-os-context context)))
