(in-package #:lisp-sentry)


(defgeneric dump-context (context))

(define-json-object (contexts-interface)
  "The [Contexts Interface](https://develop.sentry.dev/sdk/data-model/event-payloads/contexts)
 provides additional context data. Typically, this is data related to the
current user and the environment. For example, the device or application
version.

See `MAKE-CONTEXTS-INTERFACE`, `ADD-CONTEXT`, `ADD-CONTEXTS`,
`MAKE-DEFAULT-CONTEXTS`"
  (contexts nil :type list :raw t :format "~{~s:~a~^,~}"
                :processor (lambda (contexts)
                             (mapcan #'dump-context contexts))
                :documentation "a list of context objects."))

(defun add-contexts (event &rest contexts)
  "Sets context list of given `EVENT` to given `CONTEXTS`."
  (push
   (make-contexts-interface :contexts contexts)
   (event-interfaces event))
  event)

(defun add-context (event context)
  "Appends given `CONTEXT` to a list of `EVENT` contexts."
  (let ((interface (find-if #'contexts-interface-p (event-interfaces event))))
    (if interface
        (push context (contexts-interface-contexts interface))
        (add-contexts event context))
    event))

(defmethod dump-event-interface ((interface contexts-interface))
  `("contexts"
    ,(dump-contexts-interface interface)))
