(in-package #:lisp-sentry)


(define-json-object (debug-meta-image :ctor %make-debug-meta-image)
  "[Debug image](https://develop.sentry.dev/sdk/data-model/event-payloads/debugmeta/#debug-images)
is dynamic library loaded into process.

See `MAKE-DEBUG-META-IMAGE`"
  (type #+linux "elf" #+darwin "macho" #+windows "pe"
        :type string
        :documentation "type of the debug image. Defaults to the correct image
type for the platform the system is compiled in.")
  (image-addr nil :type non-negative-fixnum :optional t :format "\"0x~x\""
                  :documentation "memory address, at which the image is mounted
in the virtual address space of the process.")
  (debug-id
   ""
   :type string
   :documentation "debug identifier of the dynamic library or executable.")
  (code-file
   #P""
   :type pathname :processor 'namestring
   :documentation "the absolute path to the dynamic library or executable."))

(defun make-debug-meta-image (library)
  "Creates `DEBUG-META-IMAGE` from given [CFFI](https://cffi.common-lisp.dev)
`LIBRARY`."
  (let ((path (cffi:foreign-library-pathname library))
        (id (make-array 16 :element-type '(unsigned-byte 8) :initial-element 0)))
    (declare (dynamic-extent id))
    (replace id (babel:string-to-octets (reverse (namestring path))) :end1 16)
    (%make-debug-meta-image
     :image-addr #+sbcl (#-windows sb-sys:sap-int #+windows identity
                         (sb-alien::shared-object-handle
                          (find path sb-sys:*shared-objects*
                                :key #'sb-alien::shared-object-pathname
                                :test #'equal)))
                 #-sbcl nil
     :debug-id (fuuid:to-string (fuuid:make-v5-from-octets id))
     :code-file path)))

(define-json-object (debug-meta-interface)
  "The [debug meta interface](https://develop.sentry.dev/sdk/data-model/event-payloads/debugmeta)
carries debug information for processing errors and crash reports.

See `MAKE-DEBUG-META-INTERFACE`, `ADD-DEBUG-META`"
  (images
   nil
   :type list :format "[~{~a~^,~}]"
   :processor (lambda (images)
                (mapcar #'dump-debug-meta-image images))
   :documentation "a list of dynamic libraries loaded into the process."))

(defmethod dump-event-interface ((interface debug-meta-interface))
  `("debug_meta"
    ,(dump-debug-meta-interface interface)))

(defun add-debug-meta (event &key (libraries (cffi:list-foreign-libraries)))
  "Adds `DEBUG-META-INTERFACE` with given `LIBRARIRES` list (defaulting to the
results of `CFFI:LIST-FOREIGN-LIBRARIES` call) to the given `EVENT`."
  (push
   (make-debug-meta-interface
    :images (mapcar #'make-debug-meta-image libraries))
   (event-interfaces event))
  event)
