(in-package #:lisp-sentry)


(define-json-object (request-interface)
  "The [Request interface](https://develop.sentry.dev/sdk/data-model/event-payloads/request)
contains information on a HTTP request related to the event.

See `MAKE-REQUEST-INTERFACE`, `ADD-REQUEST`"
  (method :get :type string-designator :format "\"~a\""
               :documentation "the HTTP method of the request.")
  (url nil :type string :documentation "the URL of the request if available.")
  (data nil :type string :optional t :format "~a"
            :documentation "submitted data in a format that makes the most
sense. Can be given as string or structural data of any format. **NOTE** that
this should be printed with `~A` to valid JSON string.")
  (cookies nil :type list :optional t :format "{~{~s:~s~^,~}}"
               :documentation "cookie values. Should be given as plist with
string keys and string values.")
  (env nil :type list :optional t :format "{~{\"~a\":\"~a\"~^,~}}"
           :documentation "dictionary containing environment information passed
from the server. This is where information such as Clackup env go that are not
HTTP headers. Should be given as plist with keyword keys.")
  (headers nil :type list :format "{~{\"~a\":~s~^,~}}"
               :documentation "dictionary of submitted headers. If a header
appears multiple times it, needs to be merged according to the HTTP standard for
header merging. Header names are treated case-insensitively by Sentry. Should be
given as plist with keyword or string keys and string values."))

(defun add-request (event &key (method :get) url data cookies headers env)
  "Adds `REQUEST-INTERFACE` with given `METHOD`, `URL`, `DATA`, `COOKIES`,
`HEADERS` and `ENV` to the given `EVENT`.

See `MAKE-REQUEST-INTERFACE`."
  (push
   (make-request-interface
    :method method
    :url url
    :data data
    :cookies cookies
    :headers headers
    :env env)
   (event-interfaces event))
  event)

(defmethod dump-event-interface ((interface request-interface))
  `("request"
    ,(dump-request-interface interface)))
