(in-package #:lisp-sentry)


(define-json-object (sdk-interface)
  "The [SDK Interface](https://develop.sentry.dev/sdk/data-model/event-payloads/sdk)
describes the Sentry SDK and its configuration used to capture and transmit an event.

See `MAKE-SDK-INTERFACE`, `ADD-SDK`"
  (name "lisp-sentry" :type string
                      :documentation "the name of the SDK.")
  (version #.(system-version :lisp-sentry)
           :type string
           :documentation "the version of the SDK.")
  (packages (load-time-value (get-modules
                              '("asdf" "bordeaux-threads" "cl-ppcre"
                                "drakma" "framebuffers" "local-time"
                                "machine-state" "machine-state/opengl"
                                "salza2" "trivial-mimes"))
                             t)
            :type list
            :format "[~{{\"name\":~s,\"version\":~s}~^,~}]"
            :documentation "a list of packages that were installed as part of
this SDK or the activated integrations. Each package consists of a name in the
format `source:identifier` and `version`."))

(defun add-sdk (event)
  "Adds `SDK-INTERFACE` to the given `EVENT`."
  (push
   (make-sdk-interface)
   (event-interfaces event))
  event)

(defmethod dump-event-interface ((interface sdk-interface))
  `("sdk"
    ,(dump-sdk-interface interface)))
