(in-package #:lisp-sentry)


(define-json-object (user-interface)
  "[User Interface](https://develop.sentry.dev/sdk/data-model/event-payloads/user)
is an interface describing the authenticated User for a request.

See `MAKE-USER-INTERFACE`"
  (id
   nil
   :type string :optional t
   :documentation "application-specific internal identifier for the user.")
  (username nil :type string :optional t
                :documentation "username. Typically used as a better label than
the internal id.")
  (email nil :type string :optional t
             :documentation "alternative, or addition, to the username. Sentry
is aware of email addresses and can display things such as Gravatars and unlock
messaging capabilities.")
  (ip-address "{{auto}}" :type string
                         :documentation "the user's IP address. If the user is
unauthenticated, Sentry uses the IP address as a unique identifier for the user.
Set to \"{{auto}}\" to let Sentry infer the IP address from the connection."))

(defun add-user (event &key id name email (ip-address "{{auto}}"))
  "Adds `USER-INTERFACE` with given `ID`, `NAME`, `EMAIL` and
`IP-ADDRESS` (defaulting to \"{{auto}}\") to the given `EVENT`.

You should provide at least one of `ID`, `NAME`, `EMAIL`, `IP-ADDRESS` for
Sentry to be able to tell you how many users are affected by one issue, for
example. Sending a user that has none of these attributes and only custom
attributes is valid, but not as useful."
  (push
   (make-user-interface
    :id id
    :username name
    :email email
    :ip-address ip-address)
   (event-interfaces event))
  event)

(defmethod dump-event-interface ((interface user-interface))
  `("user"
    ,(dump-user-interface interface)))
