(in-package #:lisp-sentry)


(define-json-object (attachment :ctor %make-attachment :include envelope-item)
  "[Attachment Item](https://develop.sentry.dev/sdk/data-model/envelope-items/#attachment)
contains a raw payload of an attachment file. It is always associated to an
event or transaction.

See `MAKE-ATTACHMENT`"
  (type "attachment" :type string)
  (event-id "" :type string)
  (filename #P"" :type (or pathname string) :processor 'namestring)
  (attachment-type nil :type string :optional t)
  (content-type "application/octet-stream" :type string)
  (length 0 :type array-length))

(defun make-attachment (&key envelope
                             filename
                             (type "event.attachment")
                             (content-type (mime filename)))
  "Creates `ATTACHMENT` item in the given `ENVELOPE`.

* `FILENAME` is the name of the uploaded file.
* `TYPE` is the special type of this attachment. Defaults to `event.attachment`.
* `CONTENT-TYPE` is the content type of the attachment payload. Any MIME type
may be used; defaults to the content type deduced from `FILENAME`."
  (let ((attachment (%make-attachment
                     :envelope envelope
                     :event-id (envelope-event-id envelope)
                     :filename filename
                     :attachment-type type
                     :content-type content-type)))
    (push attachment (envelope-items envelope))
    attachment))

(defmethod dump-envelope-item-payload ((item attachment))
  (let ((payload (read-file-into-byte-vector (attachment-filename item))))
    (setf (attachment-length item) (length payload))
    payload))

(defmethod dump-envelope-item-header ((item attachment) payload)
  (babel:string-to-octets
   (escape-nonprintable
    (dump-attachment item))))
