(in-package #:lisp-sentry)


(defun smaller-than-4k-p (str)
  (<= (babel:string-size-in-octets str)
      4096))

(deftype comments-string () '(and string (satisfies smaller-than-4k-p)))

(define-json-object (user-report :ctor %make-user-report :include envelope-item)
  "[User Report item](https://develop.sentry.dev/sdk/data-model/envelope-items/#user-report---deprecated)
works by associating user information and comments with an event. If both the
item and its associated event are accepted, we convert it to a user feedback.

Constraints:

* This Item may occur once per Envelope.
* User Reports can be ingested separately from their events. However, we
  recommend sending them in the same Envelope.
* You may not associate multiple User Reports to the same event.
* The event can not be more than 30 minutes old.
* If the event does not exist in the same project or was never ingested, the
  report is discarded and never converted to feedback.

See `MAKE-USER-REPORT`"
  (event-id "" :type string)
  (email nil :type string :optional t)
  (name nil :type string :optional t)
  (comments nil :type comments-string))

(defun make-user-report (&key envelope email name comments)
  "Creates `USER-REPORT` item in the given `ENVELOPE`.

* `EMAIL` is the email of the user.
* `NAME` is the name of the user.
* `COMMENTS` is the comments of the user about what happened. The max length
  is 4096 characters."
  (let ((user-report (%make-user-report
                      :envelope envelope
                      :event-id (envelope-event-id envelope)
                      :email email
                      :name name
                      :comments comments)))
    (push user-report (envelope-items envelope))
    user-report))

(defmethod dump-envelope-item-payload ((item user-report))
  (babel:string-to-octets
   (escape-nonprintable
    (dump-user-report item))))
