(in-package #:lisp-sentry)


(declaim (type hash-table *sources*))
(defvar *sources* (make-hash-table :test #'equal)
  "To display source code in Sentry stack traces, the source code should be
 saved somewhere, since the dumped Lisp image most probably wouldn't run on
the same machine it was built on. This variable holds source files saved on
build time.

See `MAKE-SAVED-SOURCES`")

(defstruct (source (:constructor %make-source))
  (lines nil :type (simple-array string)))

(defun save-source (sources file)
  (format t "LISP-SENTRY: Saving source file ~a~%" file)
  (setf (gethash (namestring file) sources)
        (make-source
         (alexandria:read-file-into-string file)))
  (values))

(defun save-sources (&key extra-dirs)
  (let ((sources (make-hash-table :test #'equal)))
    (loop :for system :in (asdf:already-loaded-systems)
          :unless (member system '("asdf" "quicklisp" "uiop") :test #'string=)
          :do (dolist (file (package-files system))
                (save-source sources file)))
    (dolist (dir extra-dirs)
      (dolist (file (directory (merge-pathnames  "**/*.lisp" (truename dir))))
        (save-source sources file)))
    sources))
