(in-package #:lisp-sentry)


(declaim (type string *dsn*))
(defvar *dsn* ""
  "Sentry [DSN](https://docs.sentry.io/concepts/key-terms/dsn-explainer) string.
If that variable is unset, the SDK will just not send any events.")

(defun send-payload (endpoint &key (method :post)
                                   (content-type "application/json")
                                   content)
  (declare (type (vector (unsigned-byte 8)) content))
  (ppcre:register-groups-bind (protocol pubkey host project)
      ("^(https?):\/\/(.+)?@(.*)\/(.*)$" *dsn* :sharedp t)
    (let* ((drakma:*text-content-types* (cons '(nil . "json")
                                              drakma:*text-content-types*))
           (body (salza2:compress-data
                  (coerce content 'salza2::octet-vector)
                  'salza2:gzip-compressor)))
      (drakma:http-request
       (format nil "~a://~a/api/~a/~a/" protocol host project endpoint)
       :method method
       :force-ssl t
       :accept "application/json"
       :additional-headers
       `(("Content-Encoding" . "gzip")
         ;; https://develop.sentry.dev/sdk/overview/#authentication
         ("X-Sentry-Auth" . ,(format nil
                                     "Sentry sentry_version=7, sentry_key=~a"
                                     pubkey)))
       :user-agent
       (format nil "lisp-sentry/~a (~a~@[ ~a~]) ~a~@[/~a~] Drakma/~a"
               #.(system-version :lisp-sentry)
               #.(or #-clisp (software-type)
                     #+windows "Windows"
                     #+linux "Linux"
                     #+macos "MacOS")
               (or #-clisp (software-version) "")
               #.(or (lisp-implementation-type) "Common Lisp")
               #.(or (lisp-implementation-version) "")
               #.(system-version :drakma))
       :content body
       :content-length (length body)
       :content-type content-type))))
